### Listing 4.1

```
application {
  secret="changeme" 
  langs="en" 
}
```

### Listing 4.2

```
<configuration>
  <conversionRule 
    conversionWord="coloredLevel"
    converterClass="play.api.Logger$ColoredLevel" />

  <appender name="STDOUT" class="ch.qos.logback.core.ConsoleAppender"> 
    <encoder>
      <pattern>
        %coloredLevel - %logger - %message%n%xException 
      </pattern>
    </encoder>
  </appender>

  <logger name="play" level="INFO" />
  <logger name="application" level="DEBUG" /> 

  <root level="ERROR">
    <appender-ref ref="STDOUT" /> 
  </root>

</configuration>
```

### Listing 4.3

```
addSbtPlugin("com.typesafe.play" % "sbt-plugin" % "2.4.3") 

addSbtPlugin("com.typesafe.sbt" % "sbt-scalariform" % "1.3.0") 
```

### Listing 4.4

```
name := "simple-vocabulary-teacher" 

version := "1.0" 

scalaVersion := "2.11.7" 

lazy val `simple-vocabulary-teacher` =
  (project in file(".")).enablePlugins(PlayScala) 

routesGenerator := InjectedRoutesGenerator 

com.typesafe.sbt.SbtScalariform.scalariformSettings 
```

### Listing 4.5


```
import javax.inject._
import play.api.http.DefaultHttpErrorHandler
import play.api._
import play.api.mvc._
import play.api.mvc.Results._
import play.api.routing.Router
import scala.concurrent._

class ErrorHandler @Inject() ( 
    env: Environment,
    config: Configuration,
    sourceMapper: OptionalSourceMapper,
    router: Provider[Router]) 
  extends DefaultHttpErrorHandler(env, config, sourceMapper, router) { 

  override protected def onNotFound( 
    request: RequestHeader, message: String
  ): Future[Result] = {
    Future.successful {
      NotFound("Could not find " + request) 
    }
  }
}
```

### Listing 4.6

```
package controllers

import play.api.mvc._

class Import extends Controller {
  def importWord(
    sourceLanguage: String,
    targetLanguage: String,
    word: String,
    translation: String
  ) = TODO
}
```

### Listing 4.7

```
PUT /import/word/:sourceLang/:word/:targetLang/:translation controllers.Import.importWord(sourceLang, word, targetLang, translation)
```

### Listing 4.8

```
import play.api.i18n.Lang
def importWord(
  sourceLanguage: Lang,
  word: String,
  targetLanguage: Lang,
  translation: String
) = TODO
```
### Listing 4.9

```
PUT /import/word/:sourceLang/:word/:targetLang/:translation controllers.Import.importWord(sourceLang: play.api.i18n.Lang, word, targetLang: play.api.i18n.Lang, translation)
```

### Listing 4.10

```
package binders

import play.api.i18n.Lang
import play.api.mvc.PathBindable

object PathBinders { 

  implicit object LangPathBindable extends PathBindable[Lang] { 
    override def bind(key: String, value: String): 
      Either[String, Lang] = 
        Lang.get(value).toRight(s"Language $value is not recognized") 

    override def unbind(key: String, value: Lang): String = value.code 
  }

}
```
### Listing 4.11

```
package models

import play.api.i18n.Lang

case class Vocabulary(
  sourceLanguage: Lang,
  targetLanguage: Lang,
  word: String,
  translation: String)
```

### Listing 4.12

```
package services

import javax.inject.Singleton
import play.api.i18n.Lang

@Singleton 
class VocabularyService {

  private var allVocabulary = List( 
    Vocabulary(Lang("en"), Lang("fr"), "hello", "bonjour"),
    Vocabulary(Lang("en"), Lang("fr"), "play", "jouer")
  )

  def addVocabulary(v: Vocabulary): Boolean = { 
    if (!allVocabulary.contains(v)) {
      allVocabulary = v :: allVocabulary
      true
    } else {
      false
    }
  }

}
```

### Listing 4.13
```
def importWord(
  sourceLanguage: Lang,
  word: String,
  targetLanguage: Lang,
  translation: String
) = Action { request => 
  val added = vocabulary.addVocabulary(
    Vocabulary(sourceLanguage, targetLanguage, word, translation)
  )
  if (added)
    Ok 
  else
    Conflict 
}
```

### Listing 4.14
```
def findRandomVocabulary(sourceLanguage: Lang, targetLanguage: Lang):
  Option[Vocabulary] = {
    Random.shuffle(allVocabulary.filter { v => 
        v.sourceLanguage == sourceLanguage &&
        v.targetLanguage == targetLanguage
      }).headOption
}

def verify(
  sourceLanguage: Lang,
  word: String,
  targetLanguage: Lang,
  translation: String): Boolean = {
    allVocabulary.contains( 
      Vocabulary(sourceLanguage, targetLanguage, word, translation)
    )
}
```

### Listing 4.15

```
GET /quiz/:sourceLang controllers.Quiz.quiz(sourceLang: play.api.i18n.Lang, targetLang: play.api.i18n.Lang)

POST /quiz/:sourceLang/check/:word controllers.Quiz.check(sourceLang: play.api.i18n.Lang, word, targetLang: play.api.i18n.Lang, translation)
```

### Listing 4.16

```
class QuizActor(out: ActorRef,
                               sourceLang: Lang,
                               targetLang: Lang,
                               vocabulary: VocabularyService)
  extends Actor { 

  private var word = "" 

  override def preStart(): Unit = sendWord() 

  def receive = {
    case translation: String
      if vocabulary.verify(
        sourceLang, word, targetLang, translation
      ) =>
        out ! "Correct!"
        sendWord() 
    case _ =>
      out ! "Incorrect, try again!"
  }

  def sendWord() = {
    vocabulary
      .findRandomVocabulary(sourceLang, targetLang).map { v =>
      out ! s"Please translate '${v.word}'"
      word = v.word 
    } getOrElse {
      out ! s"I don't know any word for ${sourceLang.code} " +
        " and ${targetLang.code}"
    }
  }
}
```

### Listing 4.17

```
object QuizActor {
  def props(out: ActorRef,
                     sourceLang: Lang,
                     targetLang: Lang,
                     vocabulary: VocabularyService): Props =
    Props(classOf[QuizActor], out, sourceLang, targetLang, vocabulary)
}
```

### Listing 4.18

```
def quizEndpoint(sourceLang: Lang, targetLang: Lang) = 
  WebSocket.acceptWithActor[String, String] {  
    request => 
      out => 
        QuizActor.props(out, sourceLang, targetLang, vocabulary) 
}
```

### Listing 4.19

```
GET   /quiz/interactive/:sourceLang/:targetLang controllers.Quiz.quizEndpoint(sourceLang: play.api.i18n.Lang, targetLang: play.api.i18n.Lang)
```

### Listing 4.20

```
import javax.inject.Inject
import play.api.http.HttpFilters
import play.filters.gzip.GzipFilter
import play.filters.headers.SecurityHeadersFilter

class Filters @Inject() (
  gzip: GzipFilter 
) extends HttpFilters { 
  val filters = Seq(gzip, SecurityHeadersFilter()) 
}
```

### Listing 4.21

```
class ScoreFilter extends Filter {
  override def apply(
    nextFilter: (RequestHeader) => Future[Result] 
  )(rh: RequestHeader): 
  Future[Result] = {

    val result = nextFilter(rh) 
    import play.api.libs.concurrent.Execution.Implicits._ 
    result.map { res =>
      if (res.header.status == 200 || res.header.status == 406) { 
        val correct = res.session(rh).get("correct").getOrElse(0)
        val wrong = res.session(rh).get("wrong").getOrElse(0)
        val score = s"\nYour current score is: $correct correct " +
          s"answers and $wrong wrong answers"
        val newBody =
          res.body andThen Enumerator(score.getBytes("UTF-8")) 
        res.copy(body = newBody) 
      } else {
        res
      }
    }
  }
}
```

### Listing 4.22

```
def check(
  sourceLanguage: Lang,
  word: String,
  targetLanguage: Lang,
  translation: String) = Action { request =>
    val isCorrect =
      vocabulary
        .verify(sourceLanguage, word, targetLanguage, translation)
    val correctScore =
      request.session.get("correct").map(_.toInt).getOrElse(0) 
    val wrongScore =
      request.session.get("wrong").map(_.toInt).getOrElse(0)
      if (isCorrect) {
        Ok.withSession( 
          "correct" -> (correctScore + 1).toString,
	"wrong" -> wrongScore.toString
        )
      } else {
        NotAcceptable.withSession(
          "correct" -> correctScore.toString,
          "wrong" -> (wrongScore + 1).toString
        )
      }
}
```
