### Listing 8.1

```
resolvers += "Typesafe repository"
  at "https://repo.typesafe.com/typesafe/releases/"

addSbtPlugin("com.typesafe.play" % "sbt-plugin" % "2.4.3") 

addSbtPlugin("com.vmunier" % "sbt-play-scalajs" % "0.2.6") 

addSbtPlugin("org.scala-js" % "sbt-scalajs" % "0.6.3") 
```

### Listing 8.2

```
lazy val scalaV = "2.11.6"

lazy val `ch08` = (project in file(".")).settings( 
  scalaVersion := scalaV,
  scalajsProjects := Seq(client), 
  pipelineStages := Seq(scalaJSProd), 
  libraryDependencies ++= Seq(
    "com.vmunier" %% "play-scalajs-scripts" % "0.2.2" 
  ),
  WebKeys.importDirectly := true 
).enablePlugins(PlayScala).dependsOn(client).aggregate(client)

lazy val client = (project in file("modules/client")).settings( 
  scalaVersion := scalaV,
  persistLauncher := true,
  persistLauncher in Test := false,
  libraryDependencies ++= Seq(
    "org.scala-js" %%% "scalajs-dom" % "0.8.0" 
  ),
  skip in packageJSDependencies := false

).enablePlugins(ScalaJSPlugin, ScalaJSPlay, SbtWeb) 
```

### Listing 8.3

```
package dashboard

import scala.scalajs.js.JSApp
import org.scalajs.dom._ 

object DashboardApp extends JSApp { 
  def main(): Unit = { 
    document.getElementById("scalajs").innerHTML = 
      "Hello form Scala.js!"
  }
}
```

### Listing 8.4

```
package dashboard

import biz.enef.angulate.ext.{Route, RouteProvider}
import biz.enef.angulate._
import scala.scalajs.js.JSApp

object DashboardApp extends JSApp {
  def main(): Unit = {
    val module = angular.createModule("dashboard", Seq("ngRoute")) 
    module.controllerOf[DashboardCtrl] 
    module.config { ($routeProvider: RouteProvider) =>
      $routeProvider  
        .when("/dashboard", Route(
          templateUrl = "/assets/partials/dashboard.html",
          controller = "dashboard.DashboardCtrl")
        ).otherwise(Route(redirectTo = "/dashboard"))
    }
  }
}
```

### Listing 8.5

```
package dashboard

import biz.enef.angulate._
import org.scalajs.dom._
import scalajs.js.Dynamic

class DashboardCtrl($scope: Dynamic) 
  extends ScopeController { 
    $scope.hello = "Hello, world" 
    $scope.helloBack = () => console.log("Hi") 
}
```

### Listing 8.6

```
<div>
    <h1>{{ hello }}</h1> 
    <button type="button" ng-click="helloBack()">Hello back</button> 
</div>
```

### Listing 8.7

```
@main("Twitter SMS service dashboard") {
  <div class="container" ng-view>
  </div>
}
```

### Listing 8.8

```
package dashboard

import biz.enef.angulate.core.{HttpPromise, ProvidedService}
import org.scalajs.dom._
import scala.scalajs.js
import scala.scalajs.js.UndefOr

trait WebsocketService extends ProvidedService { 
  def apply( 
    url: String,
    options: UndefOr[Dynamic] = js.undefined
  ): WebsocketDataStream = js.native
}

trait WebsocketDataStream extends js.Object { 
  def send[T](data: js.Any): HttpPromise[T] = js.native 
  def onMessage(
    callback: js.Function1[MessageEvent, Unit], 
    options: UndefOr[js.Dynamic] = js.undefined): Unit = js.native 
}
```

### Listing 8.9

```
package dashboard

import biz.enef.angulate._
import org.scalajs.dom._
import scala.scalajs.js.{Dynamic, JSON}
import scala.collection._

class GraphDataService($websocket: WebsocketService) extends Service { 
  val dataStream = $websocket("ws://localhost:9000/graphs") 

  private val callbacks =
    mutable.Map.empty[GraphType.Value, Dynamic => Unit] 

  def fetchGraph( 
    graphType: GraphType.Value,
    callback: Dynamic => Unit
  ) = {
    callbacks += graphType -> callback
    dataStream.send(graphType.toString)
  }

  dataStream.onMessage { (event: MessageEvent) =>
    val json: Dynamic = JSON.parse(event.data.toString)
    val graphType = GraphType.withName(json.graph_type.toString) 
    callbacks.get(graphType).map { callback => 
      callback(json)
    } getOrElse {
      console.log(s"Unknown graph type $graphType")
    }
  }
}

object GraphType extends Enumeration { 
  val MonthlySubscriptions = Value
}
```

### Listing 8.10

```
<div>
    <canvas id="line"
        class="chart chart-line"
        data="monthlySubscriptions.data"
        labels="monthlySubscriptions.labels"
        series="monthlySubscriptions.series"
        legend="true">
    </canvas>
</div>
```

### Listing 8.11

```
package services

import biz.enef.angulate.core.HttpPromise
import dashboard._
import org.scalajs.dom._
import utest._
import scala.scalajs.js
import scala.scalajs.js.UndefOr
import scala.scalajs.js.annotation.JSExportAll

object GraphDataServiceSuite extends TestSuite { 
  val tests = TestSuite {
    "GraphDataService should initialize a WebSocket connection" - { 
      val growlMock = new GrowlServiceMock
      val mockedWebsocketDataStream = new WebsocketDataStreamMock()
      val mockedWebsocketService: js.Function = { 
        (url: String, options: js.UndefOr[js.Dynamic]) =>
          mockedWebsocketDataStream.asInstanceOf[WebsocketDataStream]
      }

      new GraphDataService( 
        mockedWebsocketService.asInstanceOf[WebsocketService],
        growlMock.asInstanceOf[GrowlService]
      )

      assert(mockedWebsocketDataStream.isInitialized) 
    }
  }
}

@JSExportAll 
class GrowlServiceMock

@JSExportAll 
class WebsocketDataStreamMock {
  val isInitialized = true
  def send[T](data: js.Any): HttpPromise[T] = ???
  def onMessage(
    callback: js.Function1[MessageEvent, Unit],
    options: UndefOr[js.Dynamic] = js.undefined
  ): Unit = {}
  def onClose(callback: js.Function1[CloseEvent, Unit]): Unit = {}
  def onOpen(callback: js.Function1[js.Dynamic, Unit]): Unit = {}
}
```
