### Listing 10.1

```
$(document).ready(function () {
    $('#button').on('click', function () {
        $('#text').text('Hello');
    });
});
```

### Listing 10.2

```
@(message: String)
@main(message) {
    <button id="button">Click me</button>
    <div id="text"></div>
}
```

### Listing 10.3

```
import org.scalatest._
import play.api.test._
import play.api.test.Helpers._
import org.scalatestplus.play._

class ApplicationSpec
  extends PlaySpec 
  with OneServerPerSuite 
  with OneBrowserPerSuite 
  with FirefoxFactory { 
  "The Application" must {
    "display a text when clicking on a button" in {
      go to (s"http://localhost:$port") 
      pageTitle mustBe "Hello"
      click on find(id("button")).value 
      eventually { 
        val expectedText = app.configuration.getString("text")
        find(id("text")).map(_.text) mustBe expectedText 
      }
    }
  }
}

```

### Listing 10.4

```
name := """ch10"""

version := "1.0-SNAPSHOT"

lazy val root = (project in file(".")).enablePlugins(
  PlayScala 
)

scalaVersion := "2.11.7"

libraryDependencies ++= Seq(
  "org.webjars" %% "webjars-play" % "2.4.0-1",
  "org.webjars" % "jquery" % "2.1.4",
  "org.scalatest" %% "scalatest" % "2.2.1" % "test",
  "org.scalatestplus" %% "play" % "1.4.0-M4" % "test"
)

routesGenerator := InjectedRoutesGenerator

pipelineStages := Seq(rjs) 

RjsKeys.mainModule := "application" 

RjsKeys.mainConfig := "application" 

```

### Listing 10.5

```
(function (requirejs) { 
    'use strict';
    requirejs.config({ 
        shim: {
            'jsRoutes': {
                deps: [],
                exports: 'jsRoutes' 
            }
        },
        paths: {
            'jquery': ['../lib/jquery/jquery'] 
        }
    });
    requirejs.onError = function (err) { 
        console.log(err);
    };
    require(['jquery'], function ($) { 
        $(document).ready(function () {
            // ...
        });
    });
})(requirejs);
```

### Listing 10.6

```
docker run
  --name chapter10-jenkins 
  -p 8080:8080 
  -v ~/reactive-web-applications/CH10:/var/jenkins_home/ch10 
  docker-jenkins 
```

### Listing 10.7

```
maintainer := "John Doe <john@doe.com>" 

packageSummary in Linux := "Chapter 10 of Reactive Web Applications" 

packageDescription := 
"This package installs the Play Application used as an example
  in Chapter 10 of the book Reactive Web Applications"

serverLoading in Debian := ServerLoader.Systemd 

```
